var __defProp = Object.defineProperty;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};

// src/core/directive.ts
function defineDirective(factory) {
  return (userOptions) => {
    return factory(userOptions);
  };
}

// src/core/directives/if.ts
import process from "process";
function resolveConditional(test, env = process.env) {
  test = test || "true";
  test = test.trim();
  test = test.replace(/([^=!])=([^=])/g, "$1==$2");
  const evaluateCondition = new Function("env", `with (env){ return ( ${test} ) }`);
  try {
    return evaluateCondition(env);
  } catch (error) {
    if (error instanceof ReferenceError) {
      const match = /(\w*?) is not defined/g.exec(error.message);
      if (match && match[1]) {
        const name = match[1];
        env[name] = false;
        return resolveConditional(test, env);
      }
    }
    return false;
  }
}
var if_default = defineDirective(() => ({
  name: "#if",
  nested: true,
  pattern: {
    start: /.*?#if\s([\w !=&|()'"]*).*[\r\n]{1,2}/gm,
    end: /\s*.*?#endif.*?$/gm
  },
  processor({ matchGroup, replace, ctx }) {
    var _a;
    const code = replace(matchGroup.match);
    const regex = /.*?(#el(?:if|se))\s?([\w !=&|()'"]*).*[\r\n]{1,2}/gm;
    const codeBlock = [
      "#if",
      ((_a = matchGroup.left) == null ? void 0 : _a[1]) || "",
      ...ctx.XRegExp.split(code, regex)
    ].map((v) => v.trim());
    while (codeBlock.length) {
      const [variant, conditional, block] = codeBlock.splice(0, 3);
      if (variant === "#if" || variant === "#elif") {
        if (resolveConditional(conditional, ctx.env))
          return block;
      } else if (variant === "#else") {
        return block;
      }
    }
    return "";
  }
}));

// src/core/directives/define.ts
var define_default = defineDirective(() => ({
  nested: false,
  name: "#define",
  pattern: new RegExp(".*?#(?<directive>(?:un)?def(?:ine)?)\\s*(?<key>[\\w]*)\\s", "gm"),
  processor({ ctx }) {
    return (...args) => {
      const group = args[args.length - 1];
      if (group.directive === "define")
        ctx.env[group.key] = true;
      else if (group.directive === "undef")
        delete ctx.env[group.key];
      return "";
    };
  }
}));

// src/core/directives/error.ts
import { relative } from "path";
var error_default = defineDirective(() => ({
  nested: false,
  name: "#error",
  pattern: new RegExp(".*?#(?<directive>(?:warning)|(?:error)|(?:info))\\s*(?<message>[\\u4E00-\\u9FA5 \\t\\w]*).*?$", "gm"),
  processor({ ctx, id }) {
    return (...args) => {
      const group = args[args.length - 1];
      const source = args[args.length - 2];
      const index = args[args.length - 3];
      const lineNumber = (source.substring(0, index).match(/\n/g) || []).length + 1;
      const file = `${relative(ctx.cwd, id)}:${lineNumber}`;
      if (group.directive === "warning")
        group.directive = "warn";
      ctx.logger[group.directive](`${file}: ${group.message}`, { timestamp: true });
      return "";
    };
  }
}));

// src/core/directives/index.ts
var builtinDirectives = [define_default(), if_default(), error_default()];

// src/core/unplugin.ts
import { createUnplugin } from "unplugin";

// src/core/context.ts
import process2 from "process";
import MagicString from "magic-string";
import { createFilter, createLogger, loadEnv } from "vite";
import XRegExp from "xregexp";
function sortUserDirectives(directives) {
  const preDirectives = [];
  const postDirectives = [];
  const normalDirectives = [];
  if (directives) {
    directives.forEach((p) => {
      if (p.enforce === "pre")
        preDirectives.push(p);
      else if (p.enforce === "post")
        postDirectives.push(p);
      else
        normalDirectives.push(p);
    });
  }
  return [preDirectives, normalDirectives, postDirectives];
}
var Context = class {
  constructor({ cwd = process2.cwd(), directives = [], exclude = [/[\\/]node_modules[\\/]/, /[\\/]\.git[\\/]/], include = ["**/*"] } = {}) {
    this.directives = [];
    this.cwd = cwd;
    this.XRegExp = XRegExp;
    this.env = loadEnv(
      process2.env.NODE_ENV || "development",
      cwd,
      ""
    );
    if (directives.length === 0) {
      directives = [...builtinDirectives.map(
        (v) => typeof v === "function" ? v(this) : v
      )];
    }
    const [preDirectives, normalDirectives, postDirectives] = sortUserDirectives(directives.map(
      (v) => typeof v === "function" ? v(this) : v
    ));
    this.directives = [...preDirectives, ...normalDirectives, ...postDirectives];
    this.logger = createLogger(void 0, {
      prefix: "unplugin-preprocessor-directives"
    });
    this.directiveLoggers = this.directives.reduce((acc, directive) => {
      acc[directive.name] = createLogger(void 0, {
        prefix: `unplugin-preprocessor-directives:${directive.name}`
      });
      return acc;
    }, {});
    this.filter = createFilter(include, exclude);
  }
  replace({ code, id, directive: _directive }) {
    const directive = _directive;
    return this.XRegExp.replace(code, directive.pattern, directive.processor({
      id,
      code,
      directive,
      ctx: this
    }));
  }
  replaceRecursive({
    code,
    id,
    directive: _directive
  }) {
    const directive = _directive;
    const startRegex = new RegExp(directive.pattern.start, "mi");
    const endRegex = new RegExp(directive.pattern.end, "mi");
    const replace = (code2) => {
      const matches = XRegExp.matchRecursive(
        code2,
        startRegex.source,
        endRegex.source,
        "gmi",
        {
          valueNames: ["between", "left", "match", "right"]
        }
      );
      let builder = "";
      const matchGroup = {
        left: null,
        match: "",
        right: null
      };
      if (!matches.length)
        builder += code2;
      matches.forEach((match) => {
        switch (match.name) {
          case "between":
            builder += match.value;
            break;
          case "left":
            matchGroup.left = startRegex.exec(match.value);
            break;
          case "match":
            matchGroup.match = match.value;
            break;
          case "right":
            matchGroup.right = endRegex.exec(match.value);
            builder += directive.processor({
              id,
              code: code2,
              directive,
              ctx: this,
              replace,
              matchGroup
            });
            break;
        }
      });
      return builder;
    };
    return replace(code);
  }
  createSource(code, id) {
    return new MagicString(code, {
      filename: id
    });
  }
  transform(code, id) {
    if (!this.filter(id))
      return;
    const source = this.createSource(code, id);
    const data = this.directives.reduce((acc, directive) => {
      const { exclude = [], include = ["**/*"] } = directive;
      const filter = createFilter(include, exclude);
      if (!filter(id))
        return acc;
      try {
        acc = directive.nested ? this.replaceRecursive({
          code,
          id,
          directive
        }) : this.replace({
          code: acc,
          id,
          directive
        });
        return acc;
      } catch (error) {
        this.directiveLoggers[directive.name].error(error);
        return acc;
      }
    }, code);
    if (data !== code)
      source.overwrite(0, source.length(), data);
    if (source.hasChanged()) {
      return {
        code: source.toString(),
        map: source.generateMap({
          source: id,
          file: `${id}.map`,
          includeContent: true
        })
      };
    }
  }
};

// src/core/unplugin.ts
var unpluginFactory = (options) => {
  const ctx = new Context(options);
  return {
    name: "unplugin-preprocessor-directives",
    enforce: "pre",
    transform: (code, id) => ctx.transform(code, id),
    vite: {
      configResolved(config) {
        ctx.env = __spreadValues(__spreadValues({}, ctx.env), config.env);
      }
    }
  };
};
var unplugin = /* @__PURE__ */ createUnplugin(unpluginFactory);
var unplugin_default = unplugin;

export {
  defineDirective,
  if_default,
  define_default,
  error_default,
  builtinDirectives,
  unpluginFactory,
  unplugin_default
};
