import XRegExp, { ReplacementValue } from 'xregexp';
import { Logger, FilterPattern } from 'vite';
import * as MagicString from 'magic-string';
import MagicString__default from 'magic-string';

interface replaceOptions {
    code: string;
    id: string;
    directive: Directive;
}
declare class Context {
    cwd: string;
    XRegExp: typeof XRegExp;
    env: Record<string, string>;
    directives: Directive[];
    logger: Logger;
    directiveLoggers: Record<string, Logger>;
    filter: (id: unknown) => boolean;
    constructor({ cwd, directives, exclude, include }?: UserOptions);
    replace({ code, id, directive: _directive }: replaceOptions): string;
    replaceRecursive({ code, id, directive: _directive, }: replaceOptions): string;
    createSource(code: string, id: string): MagicString__default;
    transform(code: string, id: string): {
        code: string;
        map: MagicString.SourceMap;
    } | undefined;
}

interface MatchGroup {
    left: RegExpExecArray | null;
    match: string;
    right: RegExpExecArray | null;
}
interface BaseDirective {
    name: string;
    enforce?: 'pre' | 'post';
    nested?: boolean;
    include?: FilterPattern;
    exclude?: FilterPattern;
}
interface _ProcessorOptions {
    code: string;
    id: string;
    directive: Directive;
    ctx: Context;
}
interface NormalProcessorOptions extends _ProcessorOptions {
    directive: NormalDirective;
}
interface RecursiveProcessorOptions extends _ProcessorOptions {
    directive: RecursiveDirective;
    matchGroup: MatchGroup;
    replace: (code: string) => string;
}
interface RecursiveDirective extends BaseDirective {
    nested: true;
    pattern: {
        start: string | RegExp;
        end: string | RegExp;
    };
    processor(options: RecursiveProcessorOptions): string;
}
interface NormalDirective extends BaseDirective {
    nested?: false;
    pattern: string | RegExp;
    processor(options: NormalProcessorOptions): ReplacementValue;
}
type Directive = RecursiveDirective | NormalDirective;
type FunctionDirective = ((ctx: DirectiveContext) => Directive);
type Plugin = Directive;
type DirectiveContext = Context;
interface Options {
    cwd: string;
    directives: (Directive | FunctionDirective)[];
    include: FilterPattern;
    exclude: FilterPattern;
}
interface UserOptions extends Partial<Options> {
}
type DirectiveFactory<UserOptions> = (options: UserOptions) => FunctionDirective | Directive;

export { Directive, DirectiveContext, DirectiveFactory, FunctionDirective, MatchGroup, NormalDirective, NormalProcessorOptions, Options, Plugin, RecursiveDirective, RecursiveProcessorOptions, UserOptions, _ProcessorOptions };
